
//+------------------------------------------------------------------+
//|                                      Williams? Percent Range.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"




#property indicator_separate_window
#property indicator_minimum    -100
#property indicator_maximum    0
#property indicator_buffers    1
#property indicator_color1     DodgerBlue
#property indicator_level1     -90
#property indicator_level2     -70
#property indicator_level3     -50
#property indicator_level4     -30
#property indicator_level5     -10
#property indicator_levelcolor MidnightBlue




extern string TimeFrame               = "Current time frame";
extern int    WprPeriod               = 14;
extern int    BuyLine1                = -90;
extern int    BuyLine2                = -70; 
extern int    SellLine2               = -30;
extern int    SellLine1               = -10;                    
extern bool   Interpolate             = true;

extern bool   alertsOn                = true;
extern bool   alertsOnUpLevel1Cross   = true;
extern bool   alertsOnUpLevel2Cross   = true;
extern bool   alertsOnDownLevel1Cross = true;
extern bool   alertsOnDownLevel2Cross = true;
extern bool   alertsOnCurrent         = true;
extern bool   alertsMessage           = true;
extern bool   alertsSound             = true;
extern bool   alertsEmail             = false;





double wpr[];



string IndicatorFileName;
bool   Calculatewpr;
bool   ReturningBars;
int    timeFrame;





int init()
  {
  
   SetIndexBuffer(0,wpr);
   
   
   IndicatorFileName = WindowExpertName();
   Calculatewpr      = (TimeFrame=="Calculatewpr"); if (Calculatewpr) return(0);
   ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   
   
   
   IndicatorShortName(timeFrameToString(timeFrame)+ " %R (" +WprPeriod+ ")");


   return(0);
  }
  
  
  
  
  
  
int deinit(){return(0); }



double trends[][5];
#define _tup1 0
#define _tup2 1
#define _tmi  2
#define _tdn1 3
#define _tdn2 4




int start()
  {
   int counted_bars=IndicatorCounted();
   int i,r,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { wpr[0] = limit+1; return(0); }
           
   
   
      if (Calculatewpr || timeFrame==Period())
      {
      if (ArrayRange(trends,0)!=Bars) {  ArrayResize(trends,Bars); }

       
       
       
          
      for(i = limit, r=Bars-i-1; i>=0; i--,r++)
      {
      double dMaxHigh = High[Highest(NULL, 0, MODE_HIGH,WprPeriod, i)];
      double dMinLow  = Low[Lowest(NULL,   0, MODE_LOW, WprPeriod, i)];      
              wpr[i]  = -100*(dMaxHigh - Close[i]) / (dMaxHigh - dMinLow);
              
      }
      setTrends(i,r);
      manageAlerts();
      return(0);
   }
           
    
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      if (ArrayRange(trends,0)!=Bars) ArrayResize(trends,Bars);
      
      
      for(i = limit, r=Bars-i-1; i>=0; i--,r++)
      {
      int y  = iBarShift(NULL,timeFrame,Time[i]);
      wpr[i] = iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",WprPeriod,BuyLine1,BuyLine2,SellLine2,SellLine1,0,y);

      setTrends(i,r);
      
      if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         

         datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int k = 1; k < n; k++)
         {
  	      wpr[i+k] = k*factor*wpr[i+n] + (1.0-k*factor)*wpr[i];
  	      }  	            
   }
      
   manageAlerts();
      
   return(0);
}



void setTrends(int i, int r)
{


   trends[r][_tmi]  = trends[r-1][_tmi];
   trends[r][_tup1] = trends[r-1][_tup1];
   trends[r][_tup2] = trends[r-1][_tup2];
   trends[r][_tdn1] = trends[r-1][_tdn1];
   trends[r][_tdn2] = trends[r-1][_tdn2];
   
      if (wpr[i] > SellLine2) trends[r][_tdn2] =  1;
      if (wpr[i] < SellLine2) trends[r][_tdn2] = -1;
      if (wpr[i] > SellLine1) trends[r][_tdn1] =  1;
      if (wpr[i] < SellLine1) trends[r][_tdn1] = -1;
      if (wpr[i] > BuyLine2)  trends[r][_tup2] =  1;
      if (wpr[i] < BuyLine2)  trends[r][_tup2] = -1;
      if (wpr[i] > BuyLine1)  trends[r][_tup1] =  1;
      if (wpr[i] < BuyLine1)  trends[r][_tup1] = -1;
}


//
//
//
//
//

void manageAlerts()
{
   if (!Calculatewpr && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar)); 
                             whichBar = Bars-whichBar-1;

      //
      //
      //
      //
      //
            
      static datetime time1 = 0;
      static string   mess1 = "";
      if (alertsOnUpLevel1Cross && trends[whichBar][_tup1] != trends[whichBar-1][_tup1])
      {
         if (trends[whichBar][_tup1] ==  1) doAlert(time1,mess1,whichBar,"crossed Buy line 1 up");
         if (trends[whichBar][_tup1] == -1) doAlert(time1,mess1,whichBar,"crossed Buy line 1 down");
      }
      static datetime time2 = 0;
      static string   mess2 = "";
      if (alertsOnUpLevel2Cross && trends[whichBar][_tup2] != trends[whichBar-1][_tup2])
      {
         if (trends[whichBar][_tup2] ==  1) doAlert(time2,mess2,whichBar,"crossed Buy line 2 up");
         if (trends[whichBar][_tup2] == -1) doAlert(time2,mess2,whichBar,"crossed Buy line 2 down");
      }
      static datetime time3 = 0;
      static string   mess3 = "";
      if (alertsOnDownLevel1Cross && trends[whichBar][_tdn1] != trends[whichBar-1][_tdn1])
      {
         if (trends[whichBar][_tdn1] ==  1) doAlert(time3,mess3,whichBar,"crossed Sell line 1 up");
         if (trends[whichBar][_tdn1] == -1) doAlert(time3,mess3,whichBar,"crossed Sell line 1 down");
      }
      static datetime time4 = 0;
      static string   mess4 = "";
      if (alertsOnDownLevel2Cross && trends[whichBar][_tdn2] != trends[whichBar-1][_tdn2])
      {
         if (trends[whichBar][_tdn2] ==  1) doAlert(time4,mess4,whichBar,"crossed Sell line 2 up");
         if (trends[whichBar][_tdn2] == -1) doAlert(time4,mess4,whichBar,"crossed Sell line 2 down");
      }
      
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Wpr ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Wpr "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}


string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


